package de.fraunhofer.iem.dataprovider.kpi.dto

import de.fraunhofer.iem.dataprovider.kpi.enumeration.KpiKind
import de.fraunhofer.iem.dataprovider.kpi.strategy.KPICalculationStrategy


class KpiCalculationDto(
    val kind: KpiKind,
    private val calculationStrategy: KPICalculationStrategy? = null,
    private var isEmpty: Boolean = true,
    private var value: Int? = null
) {
    private val hierarchyEdges: MutableList<KPIHierarchyEdgeDto> = mutableListOf()

    fun hasChildren(): Boolean {
        return hierarchyEdges.isNotEmpty()
    }

    fun getHierarchyEdges(): List<KPIHierarchyEdgeDto> {
        return hierarchyEdges.toList()
    }

    fun isEmpty(): Boolean {
        return isEmpty
    }

    fun calculateKPI() {
        if (calculationStrategy != null) {
            try {
                this.value = calculationStrategy.calculateKPI(this.hierarchyEdges)
                this.isEmpty = false
            } catch (exception: Exception) {
                this.isEmpty = true
                this.value = null
            }
        } else {
            this.isEmpty = true
        }
    }

    fun getValue(): Int {
        if (value != null && !isEmpty) {
            return value as Int
        } else {
            throw Exception("Value access for empty KPIs not allowed")
        }
    }

    fun addChildKPI(kpiCreateDto: KpiCalculationDto, weight: Double) {
        val hierarchyEdge = KPIHierarchyEdgeDto(this, kpiCreateDto, weight)
        this.hierarchyEdges.add(hierarchyEdge)
    }

    fun toViewModel(): KPITreeResponseDto {

        val children = this.hierarchyEdges.map {
            val child = it.to.toViewModel()
            KPITreeChildResponseDto(child, it.weight)
        }

        return this.kind.toViewModel(value = value ?: -1, children = children, isEmpty = isEmpty)
    }
}
